<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/time_tracker/credits.htm
// +----------------------------------------------------------------------+

import('ttClientHelper');
import('DateAndTime');

// Class ttInvoiceHelper is used for help with invoices.
class ttInvoiceHelper {
	
  // insert - inserts an invoice in database.
  static function insert($fields)
  {
    $mdb2 = getConnection();
    
    $team_id = (int) $fields['team_id'];
    $name = $fields['name'];
  	if (!$name) return false;
  	$client_id = (int) $fields['client_id'];

  	$date = $fields['date'];
    $discount = $fields['discount'];
    
    // Insert a new invoice record.
    $sql = "insert into tt_invoices (team_id, name, date, client_id, discount)
      values($team_id, ".$mdb2->quote($name).", ".$mdb2->quote($date).", $client_id, ".$mdb2->quote($discount).")"; 
    $affected = $mdb2->exec($sql);
    
    if (is_a($affected, 'PEAR_Error'))
      return false;

    $last_id = 0;
    $sql = "select last_insert_id() as last_insert_id";
    $res = $mdb2->query($sql);
    $val = $res->fetchRow();
    $last_id = $val['last_insert_id'];
       
  	return $last_id;
  }
	
  // getInvoice - obtains invoice data from the database.
  static function getInvoice($invoice_id) {
  	global $user;
    $mdb2 = getConnection();
    
    $sql = "select * from tt_invoices where id = $invoice_id and team_id = $user->team_id";
    $res = $mdb2->query($sql);
    if (!is_a($res, 'PEAR_Error')) {
      if ($val = $res->fetchRow())
        return $val;
    }
    return false;
  }
  
  // The getInvoiceByName looks up an invoice by name.
  static function getInvoiceByName($invoice_name) {
  	
    $mdb2 = getConnection();
    global $user;

    $sql = "select id from tt_invoices where team_id = $user->team_id and name = ".$mdb2->quote($invoice_name);
  	$res = $mdb2->query($sql);
  	if (!is_a($res, 'PEAR_Error')) {
      $val = $res->fetchRow();
	  if ($val['id']) {
        return $val;
      }
    }
    return false;
  }

  // The getInvoiceItems retrieves tt_log items associated with the invoice. 
  static function getInvoiceItems($invoice_id) {
    global $user;
    global $i18n;
    $mdb2 = getConnection();

    // At this time only detailed invoice is supported.
    // It is anticipated to support "totals only" and "daily subtotals" options later on.
    $sql = "select l.date as date, u.name as user_name, p.name as project_name, t.name as task_name,
      time_format(l.duration, '%k:%i') as duration,
      round(l.billable * upb.rate * time_to_sec(l.duration)/3600, 2) as cost from tt_log l
      inner join tt_users u on (l.user_id = u.id)
      left join tt_projects p on (p.id = l.project_id)
      left join tt_tasks t on (t.id = l.task_id)
      left join tt_user_project_binds upb on (upb.user_id = l.user_id and upb.project_id = l.project_id)
      where l.billable = 1 and l.invoice_id = $invoice_id order by l.date, u.name"; // TODO: add additional sorting if necessary here.
    $res = $mdb2->query($sql);
    if (!is_a($res, 'PEAR_Error')) {
      $dt = new DateAndTime(DB_DATEFORMAT);
      while ($val = $res->fetchRow()) {
        $dt->parseVal($val['date']);
        $val['date'] = $dt->toString($user->date_format);
        $result[] = $val;
      }
    }
    return $result;
  }
  
  // delete - deletes the invoice data from the database.
  static function delete($invoice_id) {
  	global $user;
    $mdb2 = getConnection();

    // Remove the invoice id from the associated time records.
    $sql = "update tt_log set invoice_id = NULL where invoice_id = $invoice_id";
    $affected = $mdb2->exec($sql);
  	if (is_a($affected, 'PEAR_Error'))
  	  return false;
 
  	$sql = "delete from tt_invoices where id = $invoice_id and team_id = $user->team_id";
  	$affected = $mdb2->exec($sql);
    return (!is_a($affected, 'PEAR_Error'));
  }
  
  
  // The invoiceableItemsExist determines whether invoiceable records exist in the specified period.
  static function invoiceableItemsExist($fields) {
      	
  	$mdb2 = getConnection();
    global $user;

    $client_id = (int) $fields['client_id'];
    
    $start_date = new DateAndTime($user->date_format, $fields['start_date']);
    $start = $start_date->toString(DB_DATEFORMAT);
    
    $end_date = new DateAndTime($user->date_format, $fields['end_date']);
    $end = $end_date->toString(DB_DATEFORMAT);
    
    $sql = "select count(*) as num from tt_log l, tt_user_project_binds upb
      where l.client_id = $client_id and l.invoice_id is NULL
      and l.date >= ".$mdb2->quote($start)." and l.date <= ".$mdb2->quote($end)."
      and l.billable * upb.rate * time_to_sec(l.duration)/3600 > 0
      and upb.user_id = l.user_id and upb.project_id = l.project_id";
  	$res = $mdb2->query($sql);

  	if (!is_a($res, 'PEAR_Error')) {
      $val = $res->fetchRow();
	  if ($val['num']) {
        return true;
      }
    }
    
    return false;
  }
  
  // createInvoice - marks items for invoice as belonging to it (with its reference number).
  static function createInvoice($fields) {
  	
  	$mdb2 = getConnection();
    global $user;
  	
  	$name = $fields['name'];
  	if (!$name)
  	  return false;

  	$client_id = (int) $fields['client_id'];

  	$invoice_date = new DateAndTime($user->date_format, $fields['date']);
    $date = $invoice_date->toString(DB_DATEFORMAT);
  	
    $start_date = new DateAndTime($user->date_format, $fields['start_date']);
    $start = $start_date->toString(DB_DATEFORMAT);
    
    $end_date = new DateAndTime($user->date_format, $fields['end_date']);
    $end = $end_date->toString(DB_DATEFORMAT);
    
    $discount = str_replace(',', '.', $fields['discount']);
    if ($discount == '') $discount = 0;

    // Create a new invoice record.
    $sql = "insert into tt_invoices (team_id, name, date, client_id, discount)
      values($user->team_id, ".$mdb2->quote($name).", ".$mdb2->quote($date).", $client_id, ".$mdb2->quote($discount).")"; 
    $affected = $mdb2->exec($sql);
    if (is_a($affected, 'PEAR_Error'))
      return false;

    // Mark associated invoice items with invoice id.
    $last_id = 0;
    $sql = "select last_insert_id() as last_insert_id";
    $res = $mdb2->query($sql);
    $val = $res->fetchRow();
    $last_id = $val['last_insert_id'];

    $sql = "update tt_log set invoice_id = $last_id where client_id = $client_id and invoice_id is NULL
      and date >= ".$mdb2->quote($start)." and date <= ".$mdb2->quote($end);
    $affected = $mdb2->exec($sql);
    return (!is_a($affected, 'PEAR_Error'));
  }
  
  // prepareInvoiceBody - prepares an email body for invoice.
  static function prepareInvoiceBody($invoice_id, $comment)
  {
  	global $user;
  	global $i18n;
  	
  	$invoice = ttInvoiceHelper::getInvoice($invoice_id);
  	$client = ttClientHelper::getClient($invoice['client_id'], true);
  	$invoice_items = ttInvoiceHelper::getInvoiceItems($invoice_id);
  	
  	$subtotal = 0;
    foreach($invoice_items as $item)
      $subtotal += $item['cost'];
    $discount_percent = $invoice['discount'];
    $discount = $subtotal * $discount_percent / 100;
    $tax_percent = $client['tax'];
    $tax = round(($subtotal - ($subtotal * $discount_percent / 100))* ($tax_percent / 100), 2);
    $total = round(($subtotal - ($subtotal * $discount_percent / 100))* (1 + $tax_percent / 100), 2);
    
    $subtotal = $user->currency.' '.str_replace('.', $i18n->getKey('format.float_delimiter'), sprintf('%8.2f', round($subtotal, 2)));
    if ($discount > 0) {
      $discout_percent = str_replace('.', $i18n->getKey('format.float_delimiter'), $discount_percent);
      $discount = $user->currency.' '.str_replace('.', $i18n->getKey('format.float_delimiter'), sprintf('%8.2f', round($discount, 2)));
    }
    $tax_percent = str_replace('.', $i18n->getKey('format.float_delimiter'), $tax_percent);
    $tax = $user->currency.' '.str_replace('.', $i18n->getKey('format.float_delimiter'), sprintf('%8.2f', round($tax, 2)));
    $total = $user->currency.' '.str_replace('.', $i18n->getKey('format.float_delimiter'), sprintf('%8.2f', round($total, 2)));
    
    if ('.' != $i18n->getKey('format.float_delimiter')) {
      foreach ($invoice_items as &$item) {
        $item['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), $item['cost']);
      }
      unset($item); // Unset the reference. If we don't, the foreach loop below modifies the array while printing.
                    // See http://stackoverflow.com/questions/8220399/php-foreach-pass-by-reference-last-element-duplicating-bug
    }
    
    // Define some styles to use in email.
    $style_title = 'text-align: center; font-size: 15pt; font-family: Arial, Helvetica, sans-serif;';
    $style_tableHeader = 'font-weight: bold; background-color: #a6ccf7;';
    
    // Start creating email body.
    $body = '<html>';
    $body .= '<head><meta http-equiv="content-type" content="text/html; charset='.CHARSET.'"></head>';
    $body .= '<body>';
    
    // Output title.
    $body .= '<p style="'.$style_title.'">'.$i18n->getKey('title.invoice').' '.$invoice['name'].'</p>';
    
    // Output invoice info.
    $body .= '<table>';
    $body .= '<tr><td><b>'.$i18n->getKey('label.date').':</b> '.$invoice['date'].'</td></tr>';
    $body .= '<tr><td><b>'.$i18n->getKey('label.client').':</b> '.$client['name'].'</td></tr>';
    $body .= '<tr><td><b>'.$i18n->getKey('label.client_address').':</b> '.$client['address'].'</td></tr>';
    $body .= '</table>';
    
    $body .= '<p></p>';
    
    // Output invoice items.
    $body .= '<table border="0" cellpadding="4" cellspacing="0" width="100%">';
    $body .= '<tr>';
    $body .= '<td style="'.$style_tableHeader.'">'.$i18n->getKey('label.date').'</td>';
    $body .= '<td style="'.$style_tableHeader.'">'.$i18n->getKey('form.invoice.person').'</td>';
    if (MODE_PROJECTS == $user->tracking_mode || MODE_PROJECTS_AND_TASKS == $user->tracking_mode)
      $body .= '<td style="'.$style_tableHeader.'">'.$i18n->getKey('label.project').'</td>';
    if (MODE_PROJECTS_AND_TASKS == $user->tracking_mode)
      $body .= '<td style="'.$style_tableHeader.'">'.$i18n->getKey('label.task').'</td>';
    $body .= '<td style="'.$style_tableHeader.'" align="right">'.$i18n->getKey('label.duration').'</td>';
    $body .= '<td style="'.$style_tableHeader.'" align="right">'.$i18n->getKey('label.cost').'</td>';
    $body .= '</tr>';
    foreach ($invoice_items as $item) {
      $body .= '<tr>';
      $body .= '<td>'.$item['date'].'</td>';
      $body .= '<td>'.$item['user_name'].'</td>';
      if (MODE_PROJECTS == $user->tracking_mode || MODE_PROJECTS_AND_TASKS == $user->tracking_mode)
        $body .= '<td>'.$item['project_name'].'</td>';
      if (MODE_PROJECTS_AND_TASKS == $user->tracking_mode)
        $body .= '<td>'.$item['task_name'].'</td>';
      $body .= '<td align="right">'.$item['duration'].'</td>';
      $body .= '<td align="right">'.$item['cost'].'</td>';
      $body .= '</tr>';
    }
    // Output summary.
    $colspan = 3;
    if (MODE_PROJECTS == $user->tracking_mode)
      $colspan++;
    else if (MODE_PROJECTS_AND_TASKS == $user->tracking_mode)
      $colspan += 2;
    $body .= '<tr><td>&nbsp;</td></tr>';
    $body .= '<tr><td colspan="'.$colspan.'" align="right"><b>'.$i18n->getKey('label.subtotal').':</b></td><td align="right">'.$subtotal.'</td></tr>';
    if ($discount)
      $body .= '<tr><td colspan="'.$colspan.'" align="right"><b>'.$i18n->getKey('label.discount').' '.$discount_percent.'%:</b></td><td align="right"><nobr> - '.$discount.'</nobr></td></tr>';
    $body .= '<tr><td colspan="'.$colspan.'" align="right"><b>'.$i18n->getKey('label.tax').' '.$tax_percent.'%:</b></td><td align="right">'.$tax.'</td></tr>';
    $body .= '<tr><td colspan="'.$colspan.'" align="right"><b>'.$i18n->getKey('form.invoice.total').':</b></td><td align="right">'.$total.'</td></tr>';
    $body .= '</table>';
  
    // Output comment.
    $body .= '<p>'.$comment.'</p>';
    // Output footer.
    $body .= '<p style="text-align: center;">'.$i18n->getKey('form.mail.footer').'</p>';
    // Finish creating email body.
    $body .= '</body></html>';

    return $body;
  }
}
?>